// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef AISEARCHAGENTMTDF_H
#define AISEARCHAGENTMTDF_H

#include "AISearchAgent.h"

class Board;
class Move;

class AISearchAgentMTDF : public AISearchAgent
{
public:
  // Construction
  AISearchAgentMTDF();  

  // A measure of the effort we are willing to expend on search
  static int MaxIterations;
  static int MaxSearchSize;
  
  /****************************************************************************
   * PUBLIC METHODS
   ***************************************************************************/

  // Move selection: An iterative-deepening paradigm calling MTD(f) repeatedly
  Move PickBestMove(Board& theBoard, bool& Continue);  

  /***************************************************************************
   * PRIVATE METHODS
   **************************************************************************/
private:  
  // Use the MTDF algorithm to find a good move.  MTDF repeatedly calls
  // alphabeta with a zero-width search window, which creates very many quick
  // cutoffs.  If alphabeta fails low, the next call will place the search
  // window lower; in a sense, MTDF is a sort of binary search mechanism into
  // the minimax space.
  Move MTDF(Board& theBoard, int target, int depth, bool& Continue);
  
  // The standard alphabeta, with the top level "unrolled" so that it can
  // return a jcMove structure instead of a mere minimax value
  // See AISearchAgent.Alphabeta for detailed comments on this code
  Move UnrolledAlphabeta(Board& theBoard, int depth, int alpha, int beta, bool& Continue);  
};

#endif
