// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef ANIMATIONBLENDER_H
#define ANIMATIONBLENDER_H

class AnimationBlender 
{ 
public: 
  // enums
  enum BlendingTransition 
  { 
    BlendSwitch,         // stop source and start dest 
    BlendWhileAnimating, // cross fade, blend source animation out while blending destination animation in 
    BlendThenAnimate     // blend source to first frame of dest, when done, start dest anim 
  }; 

  // constructors
  AnimationBlender(Entity*); 
  AnimationBlender(AnimationBlender* copy);

  // methods
  void init (const String &animation, const bool loop = false); 
  void blend (const String&, BlendingTransition, Real, const bool loop = false); 
  void addTime (Real); 

  // accessors
  Real getProgress() {return mTimeleft/ mDuration;}   
  AnimationState* getSource();
  AnimationState* getTarget();
    
private: 
  // data members  
  Entity *mEntity; 
  AnimationState *mSource; 
  AnimationState *mTarget; 
  Real mTimeleft, mDuration; 
  BlendingTransition mTransition;        
}; 

#endif
