// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef CHESSAPPLICATION_H
#define CHESSAPPLICATION_H

#include "Application.h"

class StateManager;
class ChessInputManager;
class GameOGRE;
class GameAI;

class ChessApplication : public Application, public RenderTargetListener
{
public:
	// Creation/destruction
	ChessApplication();
	~ChessApplication();

	void setMenuOverlay(Overlay* menuOverlay);
	bool getShowDebugOverlay();
	void setShowDebugOverlay(bool show);
  void setShowMessageOverlay(bool show);
	void updateDebugOverlay();
	void saveScreenshot();
  SceneNode* GetGameRootNode();

  void AttachRootNode();
  void DetachRootNode();
  
	// RenderTargetListener overrides
	virtual void preViewportUpdate(const RenderTargetViewportEvent &evt);
	virtual void postViewportUpdate(const RenderTargetViewportEvent &evt);
  virtual void preRenderTargetUpdate(const RenderTargetEvent& evt);
  virtual void postRenderTargetUpdate(const RenderTargetEvent& evt);

protected:
  void createViewports();	
	void createScene();

	// Application overrides
	virtual void preStartRendering();

	Camera *mPlayerCamera, *mReflectCamera, *mReflectCameraGround;  
	Viewport *mPlayerViewport, *mMenuViewport, *mMessageViewport;
	Overlay *mMenuOverlay, *mDebugOverlay, *mMessageOverlay;
	StateManager* mStateManager;
  ChessInputManager* mInputManager;
	GameOGRE* mGameOGRE;
  GameAI* mGameAI;
  SceneNode* mGameRoot;
	RenderTarget::FrameStats mLastFrameStats;
  Ogre::MovablePlane* mPlane;
	Entity *mBoard;     
  bool mShowDebugOverlay, mShowMessageOverlay;
	int mScreenshotCounter;
  bool mRootNodeAttached;
};

#endif
