// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef GAMEAI_H
#define GAMEAI_H

class Board;
class Player;
class Move;

class GameAI : public Singleton<GameAI>
{
public:
	GameAI();
	~GameAI();	

  bool InitializeGame();        
  void SetPlayer(const int colour, const int type);       
  void Start(void);
  void Pause(void);
  void EndGame(void);
  void Update(void);   

  void SetDifficulty(String difficulty);
  
  bool IsCurrentPlayerAI(void);
  int GetCurrentPlayerColour(void);
  Player* GetCurrentPlayer(void);
  Board* GetGameBoard(void);

  void CreateMove(int source, int dest);  
  void SetPromotion(String type);
  void GetCheckStatus(void);  

private:      
  bool IsRunning;  
  Player *Players[2];      
  Board *GameBoard;
  String mDifficulty;
  Move *nextMove, *testMove;    
  boost::thread* AIThread;  
  bool IsThreadRunning;    
  void CreateAIThread(void);
  void DestroyAIThread(void);
  void AITurn(void);
};

#endif
