// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef HAND_H
#define HAND_H

#include "HandState.h"

class Hand
{
public:
	// Creation/destruction
  Hand(const int colour);
  ~Hand();	   

  // accessors
  int GetColour(void) {return mColour;}
  SceneManager* GetSceneManagerPtr(void);
  Entity* GetEntityPtr(void);
  SceneNode* GetSceneNodePtr(void);
  HandState* GetHandState(void);
    
  void SetAction(const HandState::ActionType state, 
                 const int Source = -1, const int Destination = -1, 
                 const int CastlingSource = -1, const int CastlingDestination = -1,
                 const String PromoType = "");  
  void ToggleParticle(bool toggle);
  void Update(const Real dt);        

  int GetNextTakeSlot(void) {return PiecesTaken++;}

  // static methods
  static Hand* GetHand(const int colour) {return mHands[colour];}
  static void CreateHands(void);
  static void DestroyHands(void);
  static void UpdateHands(const Real dt);     
  
  static bool IsInitialised;
  
protected:  
  int PiecesTaken;  
  int mColour;                     
  Entity* mEntity;
  SceneNode* mScenenode;  
  HandState* mHandState;  

private:
  void SetState(const HandState::StateType state);

  static Hand* mHands[2];
  SceneManager* mSceneManager;    
  ParticleSystem* mMagicWand;  
};

#endif
