// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef STATEMANAGER_H
#define STATEMANAGER_H

#include <OIS.h>

using namespace OIS;

class State;

class StateManager : public FrameListener, public KeyListener, public MouseListener,
                     public Singleton<StateManager>
{
public:
	// Creation/destruction
	StateManager(RenderWindow* window);
	virtual ~StateManager();
	
	void capture();

public:
	// State management
	virtual void changeState(State* state);
	virtual void pushState(State* state);
	virtual void popState();
	virtual void quit();

public:
	// FrameListener overrides
	virtual bool frameStarted(const FrameEvent &evt);
	virtual bool frameEnded(const FrameEvent &evt);

public:
	// KeyListener overrides
	virtual bool keyPressed(const KeyEvent &e);
	virtual bool keyReleased(const KeyEvent &e);

	void setKeyRepeatDelay(Real initial = 0.15, Real continous = 0.05);

public:
	// MouseListener overrides
	virtual bool mousePressed(const MouseEvent &e, MouseButtonID id );
	virtual bool mouseReleased(const MouseEvent &e, MouseButtonID id );
	virtual bool mouseMoved(const MouseEvent &e);

	Keyboard *mKeyboard;

protected:
	RenderWindow* mWindow;	
	std::vector<State*> mStateStack;
	bool mRunning;
	Real mInitialRepeatKeyDelay;
	Real mContinousKeyRepeatDelay;
	int mRepeatKey;
	Real mRepeatKeyDelay;

	InputManager *m_ois;
	Mouse *mMouse;
};

#endif
