// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef TRANSPOSITIONENTRY_H
#define TRANSPOSITIONENTRY_H

class TranspositionEntry
{
public:
  // construction
  TranspositionEntry() {};

  // Data fields, beginning with the actual value of the board and whether this
  // value represents an accurate evaluation or only a boundary
  int theEvalType;
  int theEval;

  // This value was obtained through a search to what depth?  0 means that
  // it was obtained during quiescence search (which is always effectively
  // of infinite depth but only within the quiescence domain; full-width
  // search of depth 1 is still more valuable than whatever Qsearch result)
  int theDepth;

  // Board position signature, used to detect collisions
  dong theLock;

  // What this entry stored so long ago that it may no longer be useful?
  // Without this, the table will slowly become clogged with old, deep search
  // results for positions with no chance of happening again, and new positions
  // (specifically the 0-depth quiescence search positions) will never be
  // stored!
  int timeStamp;

  static const int NULL_ENTRY = -1;    
};

#endif
