// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "AnimationBlender.h" 

AnimationState* AnimationBlender::getSource() { return mSource; } 
AnimationState* AnimationBlender::getTarget() { return mTarget; } 

AnimationBlender::AnimationBlender(Entity *entity) 
  : mEntity(entity), mSource(0), mTarget(0) {} 

// copy constructor (of sorts)
AnimationBlender::AnimationBlender(AnimationBlender* copy)
{  
  mEntity = copy->mEntity; 
  mSource = copy->mSource; 
  mTarget = copy->mTarget; 
  mTimeleft = copy->mTimeleft;
  mDuration = copy->mDuration; 
  mTransition = copy->mTransition;    
}

void AnimationBlender::init(const String &animation, const bool loop) 
{ 
   AnimationStateSet *set = mEntity->getAllAnimationStates(); 
   AnimationStateIterator it = set->getAnimationStateIterator();

   while(it.hasMoreElements()) 
   { 
      AnimationState *anim = it.getNext(); 
      anim->setEnabled(false); 
      anim->setWeight(0); 
      anim->setTimePosition(0); 
   } 

   mSource = mEntity->getAnimationState( animation ); 
   mSource->setEnabled(true); 
   mSource->setWeight(1); 
   mSource->setLoop(loop);
   mTimeleft = 0; 
   mDuration = 1; 
   mTarget = 0;    

} 

void AnimationBlender::blend(const String &animation, BlendingTransition transition, Real duration, const bool loop) 
{ 
   if( transition == AnimationBlender::BlendSwitch ) 
   { 
      if( mSource != 0 ) 
      {
         mSource->setWeight(0);
         mSource->setEnabled(false);           
      }
      mSource = mEntity->getAnimationState( animation ); 
      mSource->setEnabled(true); 
      mSource->setWeight(1); 
      mSource->setTimePosition(0); 
      mSource->setLoop(loop);
      mTimeleft = 0; 
   } 
   else 
   { 
      AnimationState *newTarget = mEntity->getAnimationState( animation ); 

      if( mTimeleft > 0 ) 
      { 
         // oops, weren't finished yet 
         if( newTarget == mTarget ) 
         { 
            // nothing to do! (ignoring duration here) 
         } 
         else if( newTarget == mSource ) 
         { 
            // going back to the source state, so let's switch 
            mSource = mTarget; 
            mTarget = newTarget; 
            mTimeleft = mDuration - mTimeleft; // i'm ignoring the new duration here 
         } 
         else 
         { 
            // ok, newTarget is really new, so either we simply replace the target with this one, or 
            // we make the target the new source 
            if( mTimeleft < mDuration * 0.5 ) 
            { 
               // simply replace the target with this one 
               mTarget->setEnabled(false); 
               mTarget->setWeight(0);               
            } 
            else 
            { 
               // old target becomes new source 
               mSource->setEnabled(false); 
               mSource->setWeight(0); 
               mSource = mTarget; 
            } 
            mTarget = newTarget; 
            mTarget->setEnabled(true); 
            mTarget->setLoop(loop);
            mTarget->setWeight( 1.0 - mTimeleft / mDuration ); 
            mTarget->setTimePosition(0); 
         } 
      } 
      else 
      {                 
         mTransition = transition; 
         mTimeleft = mDuration = duration; 
         mTarget = newTarget; 
         mTarget->setEnabled(true); 
         mTarget->setLoop(loop);
         mTarget->setWeight(0); 
         mTarget->setTimePosition(0);          
      } 
   } 
} 

void AnimationBlender::addTime( Real time ) 
{ 
   if( mSource != 0 ) 
   { 
      if( mTimeleft > 0 ) 
      { 
         mTimeleft -= time; 
         if( mTimeleft < 0 ) 
         { 
            // finish blending 
            mSource->setEnabled(false); 
            mSource->setWeight(0); 
            mSource = mTarget; 
            mSource->setEnabled(true); 
            mSource->setWeight(1); 
            mTarget = 0; 
         } 
         else 
         { 
            // still blending, advance weights 
            mSource->setWeight( mTimeleft / mDuration ); 
            mTarget->setWeight( 1.0 - mTimeleft / mDuration ); 

            if( mTransition == AnimationBlender::BlendWhileAnimating ) 
               mTarget->addTime( time ); 
         } 
      } 
      mSource->addTime( time ); 
   } 
} 
