// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "CollisionEntity.h"

CollisionModel3D* CollisionEntity::getCollisionModel(void) { return mCollisionModel; }    

void CollisionEntity::Initialise(void)
{     
  int numVertices = 0;
  int numIndices = 0;  

  // create a new collision model
  mCollisionModel = newCollisionModel3D(true);

  MeshPtr pMesh = mEntity->getMesh();

  int i = 0, j = 0, k = 0;
  for (i = 0; i < pMesh->getNumSubMeshes(); i++)
  {
    SubMesh* pSubMesh = pMesh->getSubMesh(i);        
    numVertices += pSubMesh->vertexData->vertexCount;    
    numIndices += pSubMesh->indexData->indexCount;
  }  

  // set the number of triangles
  mCollisionModel->setTriangleNumber(numIndices / 3);

  // Count the number of vertices and incides so we can Set them 
  int indexCount = 0, vertListCount = 0;  
  
  for (i = 0; i < pMesh->getNumSubMeshes(); i++)
  {
    SubMesh* pSubMesh = pMesh->getSubMesh(i);    

    const VertexElement* elem = pSubMesh->vertexData->vertexDeclaration->findElementBySemantic(VES_POSITION); 
    HardwareVertexBufferSharedPtr vbuf = pSubMesh->vertexData->vertexBufferBinding->getBuffer(elem->getSource()); 
    
    Real* test = new Real[pSubMesh->vertexData->vertexCount * 3];
    float* pDest = test;

    unsigned char* pData = static_cast<unsigned char*>(vbuf->lock(HardwareBuffer::HBL_READ_ONLY)); 

    for (j = 0; j < pSubMesh->vertexData->vertexCount; j++) 
    { 
        float* pFloat; 
        elem->baseVertexPointerToElement(pData, &pFloat); 

        *pDest++ = *pFloat++; 
        *pDest++ = *pFloat++; 
        *pDest++ = *pFloat++; 

        pData += vbuf->getVertexSize(); 
    } 

    vbuf->unlock();

    HardwareIndexBufferSharedPtr ibuf = pSubMesh->indexData->indexBuffer;
    unsigned short* test2 = new unsigned short[pSubMesh->indexData->indexCount];  

    ibuf->readData(0, ibuf->getSizeInBytes(), test2);

    float tri[3][3];               

    for (j = 0; j < pSubMesh->indexData->indexCount; j = j + 3)
    {      
      for (k = 0; k < 3; k++)
      {                  
        tri[k][0] =  test[(test2[j + k] * 3) + 0];
        tri[k][1] =  test[(test2[j + k] * 3) + 1];
        tri[k][2] =  test[(test2[j + k] * 3) + 2];
      }
      mCollisionModel->addTriangle(tri[0], tri[1], tri[2]);            
    }
    
    delete[] test;
    delete[] test2;    
  }
   
  mCollisionModel->finalize();   
}

void CollisionEntity::SetWorldTransform()
{
	Matrix4 world;
  mEntity->getParentSceneNode()->getWorldTransforms(&world);  

  float fMatrix[16];

  fMatrix[0] = world[0][0];
  fMatrix[1] = world[1][0];
  fMatrix[2] = world[2][0];
  fMatrix[3] = world[3][0];
  fMatrix[4] = world[0][1];
  fMatrix[5] = world[1][1];
  fMatrix[6] = world[2][1];
  fMatrix[7] = world[3][1];
  fMatrix[8] = world[0][2];
  fMatrix[9] = world[1][2];
  fMatrix[10] = world[2][2];
  fMatrix[11] = world[3][2];
  fMatrix[12] = world[0][3];
  fMatrix[13] = world[1][3];
  fMatrix[14] = world[2][3];
  fMatrix[15] = world[3][3]; 

  mCollisionModel->setTransform(fMatrix);  
}