// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "HistoryTable.h"
#include <algorithm>
#include "Move.h"
#include "MoveListGenerator.h"

HistoryTable HistoryTable::theInstance;

HistoryTable* HistoryTable::GetInstance()
{
  return &theInstance;
}  

// History table compilation
bool HistoryTable::AddCount(int whichPlayer, Move& mov)
{
  History[whichPlayer][mov.SourceSquare][mov.DestinationSquare]++;
  return true;
}

struct HistoryComparator : public std::binary_function<Move, Move, bool> {
  bool operator()(Move mov1, Move mov2)
  {    
    return HistoryTable::GetInstance()->History[HistoryTable::GetInstance()->CurrentHistory][ mov1.SourceSquare ][ mov1.DestinationSquare ] < 
           HistoryTable::GetInstance()->History[HistoryTable::GetInstance()->CurrentHistory][ mov2.SourceSquare ][ mov2.DestinationSquare ];
  }
};

// Sort a list of moves
bool HistoryTable::SortMoveList(MoveListGenerator& theList, int movingPlayer)
{
  CurrentHistory = movingPlayer;
  sort(theList.Moves.begin(), theList.Moves.end(), HistoryComparator());  
  return true;
}

// Once in a while, we must erase the history table to avoid ordering
// moves according to the results of very old searches
bool HistoryTable::Forget()
{
  for( int i = 0; i < 2; i++ )
    for( int j = 0; j < 64; j++ )
      for( int k = 0; k < 64; k++ )
        History[ i ][ j ][ k ] = 0;
  return true;
}