// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "MainMenuState.h"
#include "PvCPUState.h"
#include "ColourState.h"
#include "PvPState.h"
#include "OptionState.h"
#include "HelpState.h"
#include "StateManager.h"

// Singleton instance
template<> MainMenuState* Singleton<MainMenuState>::ms_Singleton = 0;

MainMenuState::MainMenuState()
{
  OverlayManager* overlayManager = OverlayManager::getSingletonPtr();

	// Set this menu's overlay  
	mMenuOverlay = static_cast<Overlay*>(overlayManager->getByName("Menu/MainMenu"));

	// Add menu items  
	mMenuItems.push_back(overlayManager->getOverlayElement("MainMenu/PvCPU"));
  mMenuItems.push_back(overlayManager->getOverlayElement("MainMenu/PvP"));
  mMenuItems.push_back(overlayManager->getOverlayElement("MainMenu/Options"));
	mMenuItems.push_back(overlayManager->getOverlayElement("MainMenu/Help"));	
	mMenuItems.push_back(overlayManager->getOverlayElement("MainMenu/Quit"));
}

void MainMenuState::onSelected(MenuItemList::iterator item)
{
	if ((*item)->getName() == "MainMenu/PvCPU")
	{
    // Start a new game
		changeState(PvCPUState::getSingletonPtr());
    // Choose a colour
    changeState(ColourState::getSingletonPtr());
	}
  else if ((*item)->getName() == "MainMenu/PvP")
	{    
    // Start a new game
		changeState(PvPState::getSingletonPtr());
    // Choose a colour
    changeState(ColourState::getSingletonPtr());
	}
  else if ((*item)->getName() == "MainMenu/Options")
	{
		// Show help
    changeState(OptionState::getSingletonPtr());
	}
	else if ((*item)->getName() == "MainMenu/Help")
	{
		// Show help
		changeState(HelpState::getSingletonPtr());
	}
	else if ((*item)->getName() == "MainMenu/Quit")
	{
		// Tell the state manager to quit
		StateManager::getSingleton().quit();
	}
}
