// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "RemoveState.h"
#include "Piece.h"
#include "Hand.h"
#include "AnimationBlender.h"

void RemoveState::Initialise(void) 
{ 
  // record this state type
  mStateType = HandState::Remove;   

  // set how long it takes for this part of the move to complete
  mStateDuration = 1;   

  // put the hand in it's starting position
  CreateMovementTrack();    

  // start the initial animation
  StartAnimation();
}

void RemoveState::CreateMovementTrack()
{       
  // get the world coordinates of where the piece is to be moved to  
  GetNextRemoveSlot(mTargetPosition.x, mTargetPosition.z);  
  mTargetPosition.y = -15;

  // tweak the offsets
  mOffset.x = 40;
  mOffset.y = 74;
  mOffset.z = 28;

  // make sure the scene node's rotation is taken into account
  Ogre::Matrix3 nodeRot;      
  mHandNode->_getDerivedOrientation().ToRotationMatrix(nodeRot);
  
  // tweak the position according to the offsets
  mTargetPosition += mOffset;  
  mOffset = mOffset * nodeRot;
  mTargetPosition = mTargetPosition * nodeRot;

  // move the moving piece's node so it's in the right place when we reattach later
  mCPSceneNode->setPosition(mTargetPosition.x - mOffset.x, -15, mTargetPosition.z - mOffset.z);

  // store the movement vector (in world coordinates) 
  // of the target position from the entities current position
  mMovementVector = mTargetPosition - mHandNode->getPosition();  

  // detach the piece from the hand
  if (mPromoType == "")
    DetachPiece(mMovingPiece, mMPSceneNode);

  // update the piece structure to its new position
  mMovingPiece->SetPosition(mDestinationSquare);
      
  // create the path to be followed     
  Animation* anim = mSceneManager->createAnimation("HandTrack" + StringConverter::toString(mHand->GetColour()), mStateDuration);

  // Spline it for nice curves
  anim->setInterpolationMode(Animation::IM_SPLINE);

  // Create a track to animate the hand
  NodeAnimationTrack* track = anim->createNodeTrack(0, mHandNode);

  // Setup keyframes
  TransformKeyFrame* key = track->createNodeKeyFrame(0); // startposition
  key->setTranslate(mHandNode->getPosition());
  key->setRotation(mHandNode->getOrientation());    
  
  key = track->createNodeKeyFrame(mStateDuration * 0.5);
  key->setTranslate(Ogre::Vector3(mTargetPosition.x - (mMovementVector.x * 0.1), mTargetPosition.y + 25, mTargetPosition.z - (mMovementVector.z * 0.1)));
  key->setRotation(mHandNode->getOrientation());    

  key = track->createNodeKeyFrame(mStateDuration);
  key->setTranslate(mTargetPosition);  
  key->setRotation(mHandNode->getOrientation()); 
}

void RemoveState::StartAnimation(void)
{
  // start the movement track
  HandState::BaseStartAnimation();

  // run the move animation
  mAnimationBlender->blend("Remove", AnimationBlender::BlendWhileAnimating, mStateDuration * 0.25, false);   
}

HandState::StateType RemoveState::Update(Real dt)
{   
  // cycle the animation
  HandState::BaseUpdate(dt);

  if (mMoveState->getTimePosition() >= 1)
  {
    mSceneManager->destroyAnimation("HandTrack" + StringConverter::toString(mHand->GetColour()));
    mMoveState = 0;
    return HandState::CaptureReturn;
  }
  else    
    return HandState::Remove;
}
