// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "ReturnState.h"
#include "Piece.h"
#include "AnimationBlender.h"
#include "Hand.h"
#include "GameAI.h"

void ReturnState::Initialise(void) 
{   
  if (mActionType == HandState::Move ||
      mActionType == HandState::Castle ||
      mActionType == HandState::EnPassant)
  {
    // record this state type
    mStateType = HandState::SimpleReturn;  

    // detach the piece from the hand
    DetachPiece(mMovingPiece, mMPSceneNode);

    // update the piece structure to its new position
    if (mActionType == HandState::EnPassant)
      if (mHand->GetColour())
        mMovingPiece->SetPosition(-1);
      else
        mMovingPiece->SetPosition(-1);
    else
      mMovingPiece->SetPosition(mDestinationSquare);
  }
  else if(mActionType == HandState::Capture)
  {
    mStateType = HandState::CaptureReturn;  
    // detach the piece from the hand
    DetachPiece(mCapturedPiece, mCPSceneNode);
    // update the piece structure to off the board
    mCapturedPiece->SetPosition(-1);
  }
  else
    mStateType = HandState::SimpleReturn;   

  // set how long it takes for this part of the move to complete
  mStateDuration = 1;

  // put the hand in it's starting position
  BaseCreateMovementTrack();  

  // set the initial animation
  StartAnimation();    
}

void ReturnState::StartAnimation(void)
{
  // start the movement track
  HandState::BaseStartAnimation();

  // run the move animation
  if (mActionType == HandState::Move ||
      mActionType == HandState::Castle ||
      mActionType == HandState::Promotion ||
      mActionType == HandState::EnPassant)
    mAnimationBlender->blend("SimpleReturn", AnimationBlender::BlendWhileAnimating, mStateDuration * 0.1, false);   
  else  
    mAnimationBlender->blend("CaptureReturn", AnimationBlender::BlendWhileAnimating, mStateDuration * 0.1, false);   
}

HandState::StateType ReturnState::Update(Real dt)
{
  // cycle the animation
  HandState::BaseUpdate(dt);  

  if (mMoveState->getTimePosition() >= 1)
  {
    // see whether this move resulted in a checkmate
    GameAI::getSingletonPtr()->GetCheckStatus();

    // we've finished the action    
    return HandState::Static;
  }
  else    
  {
    if (mActionType == HandState::Move ||
        mActionType == HandState::Castle ||
        mActionType == HandState::Promotion ||
        mActionType == HandState::EnPassant)
      return HandState::SimpleReturn;  
    else
      return HandState::CaptureReturn;      
  }
}
